@@ -1,3 +1,20 @@
+/*
+ *  Mauku Widget
+ *  Copyright (C) 2009 Henrik Hedberg <henrik.hedberg@innologies.fi>
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License version 2 as published by
+ *  the Free Software Foundation.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #define _GNU_SOURCE
 
@@ -11,14 +28,19 @@
 #include <microfeed-common/microfeedconfiguration.h>
 #include <microfeed-subscriber/microfeedsubscriber.h>
 #include <microfeed-common/microfeedmisc.h>
+#include <microfeed-common/microfeedthread.h>
 #include <microfeed-common/microfeedprotocol.h>
 #include <string.h>
 #include <dbus/dbus.h>
 #include <dbus/dbus-glib.h>
+#include <dbus/dbus-glib-lowlevel.h>
+#include <mce/dbus-names.h>
 
 #include "mauku-widget.h"
 
-#define CYCLES 30
+#define CYCLES 12
+#define DELAY 60
+#define NEXT_DELAY 6000
 #define SUBSCRIBER_ID "com.innologies.mauku_widget"
 #define HORIZONTAL_PADDING 8
 #define VERTICAL_PADDING 16
@@ -44,6 +66,8 @@ struct _Badge {
 };
 
 struct _MaukuWidgetPrivate {
+	DBusConnection* connection_system;
+	DBusConnection* connection_session;
 	MicrofeedSubscriber* subscriber;
 	Badge* current_badge;
 	GtkWidget* window;
@@ -60,8 +84,27 @@ struct _MaukuWidgetPrivate {
 	int button_open_activated;
 	cairo_surface_t* background;
 	GHashTable* image_cache;
+	gboolean visible;
 };
 
+static void open_item_in_mauku(MaukuWidget* mauku_widget, const char* publisher, const char* uri, const char* uid) {
+	DBusMessage* message;
+
+	message = dbus_message_new_method_call("com.innologies.mauku", "/com/innologies/mauku", "com.innologies.Mauku", "OpenItem");
+	dbus_message_append_args(message, DBUS_TYPE_STRING, &publisher, DBUS_TYPE_STRING, &uri, DBUS_TYPE_STRING, &uid, DBUS_TYPE_INVALID);	
+	dbus_connection_send(mauku_widget->priv->connection_system, message, NULL);
+	dbus_message_unref(message);
+}
+
+
+static void display_state_on(MaukuWidget* mauku_widget) {
+	DBusMessage* message;
+
+	message = dbus_message_new_method_call(MCE_SERVICE, MCE_REQUEST_PATH, MCE_REQUEST_IF, MCE_DISPLAY_ON_REQ);
+	dbus_connection_send(mauku_widget->priv->connection_system, message, NULL);
+	dbus_message_unref(message);
+}
+
 static Badge* badge_new(const char* publisher, time_t timestamp, const char* uid, const char* uri, const char* text, const char* info, const char* sender_image) {
 	Badge* badge;
 	
@@ -98,7 +141,7 @@ static void add_badge_into_list(MaukuWidget* mauku_widget, Badge* badge) {
 	}
 }
 
-static remove_badge_from_list(MaukuWidget* mauku_widget, Badge* badge) {
+static void remove_badge_from_list(MaukuWidget* mauku_widget, Badge* badge) {
 	if (badge->previous) {
 		badge->previous->next = badge->next;
 	}
@@ -138,22 +181,32 @@ static gboolean view_hide_show_timeout(void* user_data) {
 	if (mauku_widget->priv->pixmap) {
 		gdk_drawable_get_size(mauku_widget->priv->pixmap, &width, &height);
 
-		if (mauku_widget->priv->hide_show_counter < CYCLES) {
-			mauku_widget->priv->alpha = 1.0 - (mauku_widget->priv->hide_show_counter / (double)CYCLES);
-		} else if (mauku_widget->priv->hide_show_counter == CYCLES) {
-			mauku_widget->priv->alpha = 0.0;
-			if (mauku_widget->priv->cairo_surface) {
+		if (mauku_widget->priv->visible) {
+			if (mauku_widget->priv->hide_show_counter < CYCLES) {
+				mauku_widget->priv->alpha = 1.0 - (mauku_widget->priv->hide_show_counter / (double)CYCLES);
+			} else if (mauku_widget->priv->hide_show_counter == CYCLES) {
+				mauku_widget->priv->alpha = 0.0;
+				if (mauku_widget->priv->cairo_surface) {
+					cairo_surface_destroy(mauku_widget->priv->cairo_surface);
+					mauku_widget->priv->cairo_surface = NULL;
+				}
+			} else if (mauku_widget->priv->hide_show_counter == CYCLES * 2) {
+				mauku_widget->priv->alpha = 1.0;
+				continue_animation = FALSE;
+				mauku_widget->priv->hide_show_timeout = 0;
+			} else {
+				mauku_widget->priv->alpha = (mauku_widget->priv->hide_show_counter - CYCLES) / (double)CYCLES;
+			}
+			mauku_widget->priv->hide_show_counter++;
+		} else {
+			if (mauku_widget->priv->hide_show_counter <= CYCLES && mauku_widget->priv->cairo_surface) {
 				cairo_surface_destroy(mauku_widget->priv->cairo_surface);
 				mauku_widget->priv->cairo_surface = NULL;
+				mauku_widget->priv->alpha = 1.0;
+				continue_animation = FALSE;
+				mauku_widget->priv->hide_show_timeout = 0;
 			}
-		} else if (mauku_widget->priv->hide_show_counter == CYCLES * 2) {
-			mauku_widget->priv->alpha = 1.0;
-			continue_animation = FALSE;
-			mauku_widget->priv->hide_show_timeout = 0;
-		} else {
-			mauku_widget->priv->alpha = (mauku_widget->priv->hide_show_counter - CYCLES) / (double)CYCLES;
 		}
-		mauku_widget->priv->hide_show_counter++;
 
 		gtk_widget_queue_draw_area(GTK_WIDGET(mauku_widget), 0, 0, width, height);
 	}
@@ -164,10 +217,13 @@ static gboolean view_hide_show_timeout(void* user_data) {
 static void view_hide_show(MaukuWidget* mauku_widget) {
 	if (!mauku_widget->priv->hide_show_timeout) {
 		mauku_widget->priv->hide_show_counter = 0;
-		mauku_widget->priv->hide_show_timeout = g_timeout_add(15, view_hide_show_timeout, mauku_widget);
+		mauku_widget->priv->hide_show_timeout = g_timeout_add(DELAY, view_hide_show_timeout, mauku_widget);
 	} else if (mauku_widget->priv->hide_show_counter > CYCLES) {
 		mauku_widget->priv->hide_show_counter = (CYCLES * 2) - mauku_widget->priv->hide_show_counter;
 	}
+	if (mauku_widget->priv->visible) {
+		display_state_on(mauku_widget);
+	}
 }
 
 static gboolean view_show_next_timeout(void* user_data) {
@@ -186,7 +242,7 @@ static gboolean view_show_next_timeout(void* user_data) {
 		if (mauku_widget->priv->current_badge->previous) {
 			more_unshown = TRUE;
 			if (!mauku_widget->priv->timeout) {
-				mauku_widget->priv->timeout = g_timeout_add(5000, view_show_next_timeout, mauku_widget);
+				mauku_widget->priv->timeout = g_timeout_add(NEXT_DELAY, view_show_next_timeout, mauku_widget);
 			}
 		}
 	}
@@ -205,7 +261,7 @@ static void view_show_next(MaukuWidget* mauku_widget) {
 
 static void view_show_next_delayed(MaukuWidget* mauku_widget) {
 	if (!mauku_widget->priv->timeout) {
-		mauku_widget->priv->timeout = g_timeout_add(5000, view_show_next_timeout, mauku_widget);
+		mauku_widget->priv->timeout = g_timeout_add(NEXT_DELAY, view_show_next_timeout, mauku_widget);
 	}
 }
 
@@ -220,7 +276,7 @@ static void feed_subscribed(MicrofeedSubscriber* subscriber, const char* publish
 
 		printf("Subscribed: %s %s\n", publisher, uri);
 	
-		microfeed_subscriber_republish_items(subscriber, publisher, uri, NULL, NULL, 1, NULL, NULL);
+		microfeed_subscriber_republish_items(subscriber, publisher, uri, NULL, NULL, 50, NULL, NULL);
 	}
 }
 
@@ -361,7 +417,7 @@ static MicrofeedSubscriberCallbacks overview_callbacks = {
 	NULL
 };
 
-gboolean on_configure_event(GtkWidget* widget, GdkEventConfigure* event, gpointer user_data) {
+static gboolean on_configure_event(GtkWidget* widget, GdkEventConfigure* event, gpointer user_data) {
 	MaukuWidget* mauku_widget;
 	static int width = 0;
 	static int height = 0;
@@ -391,7 +447,7 @@ gboolean on_configure_event(GtkWidget* widget, GdkEventConfigure* event, gpointe
 	return TRUE;
 }
 
-gboolean on_expose_event(GtkWidget* widget, GdkEventExpose* event, gpointer user_data) {
+static gboolean on_expose_event(GtkWidget* widget, GdkEventExpose* event, gpointer user_data) {
 	MaukuWidget* mauku_widget;
 	int x = 0;
 	int y;
@@ -401,7 +457,6 @@ gboolean on_expose_event(GtkWidget* widget, GdkEventExpose* event, gpointer user
 	PangoLayout* layout;
 	PangoFontDescription* font_description;
 	GdkPixbuf* sender_pixbuf = NULL;
-	GdkPixbuf* content_pixbuf;
 	GdkPixbuf* pixbuf1;
 	GdkPixbuf* pixbuf2;
 
@@ -432,7 +487,7 @@ gboolean on_expose_event(GtkWidget* widget, GdkEventExpose* event, gpointer user
 				pango_layout_set_font_description(layout, font_description);
 				pango_font_description_free(font_description);
 				pango_layout_set_width(layout, (width - x - 2 * HORIZONTAL_PADDING) * PANGO_SCALE);
-				pango_layout_set_height(layout, (height - 24) * PANGO_SCALE);
+				pango_layout_set_height(layout, (height - 2* VERTICAL_PADDING - 18) * PANGO_SCALE);
 				pango_layout_set_wrap(layout, PANGO_WRAP_WORD_CHAR);
 				pango_layout_set_ellipsize(layout, PANGO_ELLIPSIZE_END);
 				pango_layout_set_text(layout, mauku_widget->priv->current_badge->text, -1);
@@ -447,7 +502,7 @@ gboolean on_expose_event(GtkWidget* widget, GdkEventExpose* event, gpointer user
 				pango_layout_set_font_description(layout, font_description);
 				pango_font_description_free(font_description);
 				pango_layout_set_width(layout, (width - x) * PANGO_SCALE);
-				pango_layout_set_height(layout, (height - y - 4) * PANGO_SCALE);
+				pango_layout_set_height(layout, (height - y - 2 - VERTICAL_PADDING) * PANGO_SCALE);
 				pango_layout_set_wrap(layout, PANGO_WRAP_WORD_CHAR);
 				pango_layout_set_ellipsize(layout, PANGO_ELLIPSIZE_END);
 				pango_layout_set_text(layout, mauku_widget->priv->current_badge->info, -1);
@@ -540,7 +595,7 @@ gboolean on_expose_event(GtkWidget* widget, GdkEventExpose* event, gpointer user
 	return TRUE;
 }
 
-gboolean on_button_press_event(GtkWidget* widget, GdkEventButton* event, gpointer user_data) {
+static gboolean on_button_press_event(GtkWidget* widget, GdkEventButton* event, gpointer user_data) {
 	MaukuWidget* mauku_widget;
 	int width;
 	int height;
@@ -567,7 +622,7 @@ gboolean on_button_press_event(GtkWidget* widget, GdkEventButton* event, gpointe
 	return FALSE;
 }
 
-gboolean on_button_release_event(GtkWidget* widget, GdkEventButton* event, gpointer user_data) {
+static gboolean on_button_release_event(GtkWidget* widget, GdkEventButton* event, gpointer user_data) {
 	MaukuWidget* mauku_widget;
 	int width;
 	int height;
@@ -596,7 +651,10 @@ gboolean on_button_release_event(GtkWidget* widget, GdkEventButton* event, gpoin
 				view_hide_show(mauku_widget);
 				view_show_next_delayed(mauku_widget);
 			} else if (event->x > 3 * width / 8 && event->x < 5 * width / 8 && mauku_widget->priv->current_badge) {
-/* TODO */
+				open_item_in_mauku(mauku_widget,
+				                   mauku_widget->priv->current_badge->publisher,
+						   mauku_widget->priv->current_badge->uri,
+						   mauku_widget->priv->current_badge->uid);
 			}
 		}
 
@@ -606,19 +664,6 @@ gboolean on_button_release_event(GtkWidget* widget, GdkEventButton* event, gpoin
 	return FALSE;
 }
 
-static int is_string_in_array(const char** array, const char* string) {
-	int retvalue = 0;
-	while (*array) {
-		if (!strcmp(*array, string)) {
-			retvalue = 1;
-			break;
-		}
-		array++;
-	}
-	
-	return retvalue;
-}
-
 static void configured_subscribe(MicrofeedSubscriber* subscriber, const char* publisher, void* user_data) {
 	microfeed_subscriber_subscribe_feed(subscriber, publisher, MICROFEED_FEED_URI_IMAGES, &images_callbacks, NULL, user_data);
 	microfeed_subscriber_subscribe_feed(subscriber, publisher, MICROFEED_FEED_URI_OVERVIEW, &overview_callbacks, feed_subscribed, user_data);
@@ -641,7 +686,6 @@ static void on_show_settings(HDHomePluginItem* hd_home_plugin_item, gpointer use
 	MaukuWidget* mauku_widget;
 	GtkWidget* about_dialog;
 	GtkWidget* label;
-	const char** subscriptions;
 	gchar* args[3];
 	GPid pid;
 	GtkWidget* message_dialog;
@@ -667,11 +711,18 @@ static void on_show_settings(HDHomePluginItem* hd_home_plugin_item, gpointer use
 	gtk_widget_destroy(about_dialog);
 }
 
+static gboolean on_visibility_notify_event(GtkWidget* widget, GdkEventVisibility* event, gpointer user_data) {
+	MaukuWidget* mauku_widget;
+	
+	mauku_widget = MAUKU_WIDGET(widget);
+	mauku_widget->priv->visible = (event->state == GDK_VISIBILITY_FULLY_OBSCURED ? FALSE : TRUE);
+	
+	return FALSE;
+}
+
 static void mauku_widget_init(MaukuWidget* mauku_widget) {
 	GdkColormap* colormap;
 	DBusError error;
-	DBusConnection* connection;
-	const char** subscriptions;
 
 	mauku_widget->priv = G_TYPE_INSTANCE_GET_PRIVATE(mauku_widget, MAUKU_TYPE_WIDGET, MaukuWidgetPrivate);
 
@@ -702,20 +753,23 @@ static void mauku_widget_init(MaukuWidget* mauku_widget) {
 	hd_home_plugin_item_set_settings(HD_HOME_PLUGIN_ITEM(mauku_widget), TRUE);
 	g_signal_connect(mauku_widget, "show-settings", G_CALLBACK(on_show_settings), NULL);
 
-	g_signal_connect(mauku_widget, "destroy", G_CALLBACK(gtk_main_quit), NULL);
 	g_signal_connect(mauku_widget, "expose_event", G_CALLBACK(on_expose_event), NULL);
 	g_signal_connect(mauku_widget, "configure_event", G_CALLBACK(on_configure_event), NULL);
 	g_signal_connect(mauku_widget, "button_press_event", G_CALLBACK(on_button_press_event), NULL);
 	g_signal_connect(mauku_widget, "motion_notify_event", G_CALLBACK(on_button_press_event), NULL);
 	g_signal_connect(mauku_widget, "button_release_event", G_CALLBACK(on_button_release_event), NULL);
-	gtk_widget_add_events(GTK_WIDGET(mauku_widget), GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_MOTION_MASK);
-	
+	g_signal_connect(mauku_widget, "visibility-notify-event", G_CALLBACK(on_visibility_notify_event), NULL);
+	gtk_widget_add_events(GTK_WIDGET(mauku_widget), GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK | GDK_BUTTON_MOTION_MASK | GDK_VISIBILITY_NOTIFY_MASK);
+
 	mauku_widget->priv->image_cache = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_object_unref);
+	mauku_widget->priv->visible = TRUE;
 
 	dbus_error_init(&error);
-	connection = dbus_bus_get(DBUS_BUS_SESSION, &error);
-	dbus_connection_setup_with_g_main(connection, NULL);
-	mauku_widget->priv->subscriber = microfeed_subscriber_new(SUBSCRIBER_ID, "/com/innologies/mauku_widget", connection);
+	mauku_widget->priv->connection_system = hd_home_plugin_item_get_dbus_connection(HD_HOME_PLUGIN_ITEM(mauku_widget), DBUS_BUS_SYSTEM, &error);
+	dbus_connection_setup_with_g_main(mauku_widget->priv->connection_system, NULL);
+	mauku_widget->priv->connection_session = hd_home_plugin_item_get_dbus_connection(HD_HOME_PLUGIN_ITEM(mauku_widget), DBUS_BUS_SESSION, &error);
+	dbus_connection_setup_with_g_main(mauku_widget->priv->connection_session, NULL);
+	mauku_widget->priv->subscriber = microfeed_subscriber_new(SUBSCRIBER_ID, "/com/innologies/mauku_widget", mauku_widget->priv->connection_session);
 	microfeed_subscriber_handle_configured_subscriptions(mauku_widget->priv->subscriber, configured_subscribe, configured_unsubscribe, mauku_widget);
 }
 
